import os

from .base import BaseProvider


class PinterestProvider(BaseProvider):
    type = 'pinterest'

    def __init__(self, **kwargs):
        self.authorization_url = 'https://api.pinterest.com/oauth/'
        self.access_token_url = 'https://api.pinterest.com/v1/oauth/token'
        super(PinterestProvider, self).__init__(**kwargs)

    def get_social_user_info(self):
        r = self.session.get('https://api.pinterest.com/v1/me/?fields=id,username,first_name,last_name')
        if r.status_code == 200:
            data = r.json().get('data')
            uid = data.get('id')

            return {
                'source_data': data,
                'uid': uid,
                'username': data.get('username', ''),
                'first_name': data.get('first_name', ''),
                'last_name': data.get('last_name', '')
            }
        return {}



from django.contrib.auth import get_user_model
from django.contrib.auth.backends import ModelBackend as StandardModelBackend

UserModel = get_user_model()


class Backend(StandardModelBackend):
    def authenticate(self, pk=None, social=False, **kwargs):
        if social:
            try:
                user = UserModel.objects.get(pk=pk)
            except UserModel.DoesNotExist:
                # Run the default password hasher once to reduce the timing
                # difference between an existing and a nonexistent user (#20760).
                UserModel().set_password(None)
            else:
                # `user_can_authenticate` only appeared in Django 1.10, so
                # ignore it if it doesn't exist for lower versions' sakes
                if not hasattr(self, 'user_can_authenticate') or self.user_can_authenticate(user):
                    return user



from .base import BaseProvider


# access tokens: 60 day expiry
class GithubProvider(BaseProvider):
    type = 'github'

    def __init__(self, **kwargs):
        self.authorization_url = 'https://github.com/login/oauth/authorize'
        self.access_token_url = 'https://github.com/login/oauth/access_token'
        super(GithubProvider, self).__init__(**kwargs)

    def get_social_user_info(self):
        r = self.session.get('https://api.github.com/user')
        if r.status_code == 200:
            data = r.json()
            uid = data.get('id')
            name = data.get('name', '')
            name_split = name.split(' ', 1)
            return {
                'source_data': data,
                'uid': uid,
                'username': data.get('login', ''),
                'email': data.get('email', ''),
                'company': data.get('company', ''),
                'organizations_url': data.get('organizations_url', ''),
                'repositories_url': data.get('repos_url', ''),
                'name': name,
                'first_name': name_split[0],
                'last_name': name_split[1] if len(name_split) > 1 else ''
            }
        return {}


from requests_oauthlib.compliance_fixes import facebook_compliance_fix

from .base import BaseProvider


# access tokens: 60 day expiry
class FacebookProvider(BaseProvider):
    type = 'facebook'

    def __init__(self, **kwargs):
        self.authorization_url = 'https://www.facebook.com/dialog/oauth'
        self.access_token_url = 'https://graph.facebook.com/oauth/access_token'
        super(FacebookProvider, self).__init__(**kwargs)
        facebook_compliance_fix(self.session)

    def get_social_user_info(self):
        r = self.session.get('https://graph.facebook.com/me')
        if r.status_code == 200:
            data = r.json()
            uid = data.get('id')
            return {
                'source_data': data,
                'uid': uid,
                'username': data.get('username', ''),
                'timezone': float(data.get('timezone', 0)),
                'gender': data.get('gender', ''),
                'language': data.get('locale', '').split('_')[0].lower(),
                'name': data.get('name', ''),
                'first_name': data.get('first_name', ''),
                'last_name': data.get('last_name', ''),
                'email': data.get('email', '')
            }
        return {}
